<?php

//  Copyright (C) 2002-2006 Balint Kis (balint@k-i-s.net)
//  Copyright (C) 2005-2010 Solomon Peachy (pizza@shaftnet.org)

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

include_once "common.php";
include_once "iptc.php";
include_once "exif.php";

function convert_rdf2sql_timestamp($date, $time) {
  $year   = substr($date,  0, 4);
  $month  = substr($date,  5, 2);
  $day    = substr($date,  8, 2);

  if ($date == "" || $year == 0 || $month == 0 || $day == 0) {
    $today = getdate();
    $year   = $today['year'];
    $month  = $today['mon'];
    $day    = $today['mday'];
  }

  $hour   = substr($time,  0, 2);
  $minute = substr($time,  3, 2);
  $second = substr($time,  6, 2);

  if ($date == "") {
    $hour   = "00";
    $minute = "00";
    $second = "00";
  }

  $sql_timestamp = $year.'-'.$month.'-'.$day.' '.$hour.':'.$minute.':'.$second;
  return $sql_timestamp;
}

function photo_parse_rdf($database, $index, $user_id, $image_data, &$output) {
  global $rdf_image_data;
  global $metering_mode_conversion;
  global $exposure_program_conversion;
  global $strings;

  $loc = "";
  $city = "";
  $state = "";
  $country = "";
  $keywords = "";
  
  $image_data = extract_exiftool_metadata($image_data, $index, "XMP", $output);
  /* Populate RDF data */
  $rdf_data = $image_data['exiftool_data_XMP'];
  foreach ($rdf_data as $key => $value) {
    $value = pg_escape_string($value);
    $image_data["rdf"] .= "<key>$key</key><value>$value</value>";
    switch ($key) {
    case "Creator": // and 'Owner' too
      $image_data["author"] = $value;
      break;
    case "Rights": // and 'Usage Terms' too?
      $image_data["copyright"] = $value;
      break;    
    case "Headline":
      $image_data["headline"] = $value;
      break;    
    case "Title":
      $image_data["title"] = $value;
      break;    
    case "Description":  // this could go into caption..?
      $image_data["caption"] = $value;
      break;    
    case "Date Created": // and 'Create Date' and 'Date'
      $image_data["date_of_exposure"] = convert_rdf2sql_timestamp($value, "00:00:00");
      break;
    case "City":
      $city = $value;
      break;
    case "State":
      $state = $value;
      break;
    case "Country":
      $country = $value;
      break;
    case "Location":
      $loc = $value;
      break;
    case "Instructions":  
      $image_data["instructions"] = $value;
      break;    
    case "Caption Writer":  
      $image_data["caption_writer"] = $value;
      break;    
    case "Web Statement":  
      $image_data["web_statement"] = $value;
      break;    
    case "Source":  
      $image_data["source"] = $value;
      break;    
    case "Credit":  
      $image_data["credit"] = $value;
      break;    
    case "Category":  
      $image_data["category"] = $value;
      break;    
    case "Transmission Reference":  
      $image_data["transmission_reference"] = $value;
      break;    
    case "Supplemental Categories":   // should get shoved onto keywords?
      $image_data["supplemental_category"] = $value;
      break;    
    case "Subject":
      $keywords = $value;
      break;
    case "F Number":
      $image_data["aperture"] = $value;
      break;
    case "Shutter Speed":
    case "Exposure Time":
      $image_data["shutter"] = convert_exif_shutter($database, $value);
      break;
    case "Exposure Compensation":
      $image_data["exp_comp"] = convert_exif_ev_comp($database, $value);
      break;
    case "Date/Time Original":
      $image_data["date_of_exposure"] = convert_exif2sql_timestamp($value);
      break;
    default:
      // print "<li>UNHANDLED: $key = $value</li>";
      break;
    }
  }

  $image_data = photo_parse_exif_orientation_colorspace($database, $index, $user_id, $image_data, "XMP");

  /* Image Comment */
  if (isset($rdf_data["User Comment"]) && !$image_data['IgnoreExifUserComment']) {
    exif_string_helper($image_data, $rdf_data, "title", "User Comment");
  }

  if (isset($rdf_data["Focal Length"])) {
    /* Focal length is special */
    $tmp = $rdf_data["Focal Length"];
    $tmp = substr($tmp, 0, strpos($tmp, 'mm'));
    
    if (isset($rdf_data["Scale Factor To 35mm Equivalent"]) && $rdf_data["Scale Factor To 35mm Equivalent"] > 2) {
      if (isset($rdf_data["Focal Length In 35mm Format"]) && $rdf_data["Focal Length In 35mm Format"]) {
	$tmp = $rdf_data["Focal Length In 35mm Format"];
	if(strpos($tmp, 'mm')) {
	  $tmp = substr($tmp, 0, strpos($tmp, 'mm'));
	}
      } else {
	$tmp = $tmp * $rdf_data["Scale Factor To 35mm Equivalent"];
      }
    }
    
    $image_data["focal_length"] = $tmp;
  }

  /* Location */
  if (($loc != "") || ($city != "") || ($state != "") || ($country != "")) {
    $image_data['location'] = 
      convert_iptc_location($database, $user_id,
                            $loc, $city, $state, $country);
  } 

  /* Keywords */
  if ($keywords != "") {
    $keys = explode(", ", $keywords);
      foreach ($keys as $key) {
	if (!$image_data['keywords'] || !in_array($key, $image_data['keywords']))
	  $image_data['keywords'][] = $key;
    }
  }

  /* 

  $rdf_image_data["lens"]                   = "null";
  $rdf_image_data["film"]                   = "null";
  $rdf_image_data["flash"]                  = "null";
  $rdf_image_data["camera_metering"]        = "null";
  $rdf_image_data["camera_program"]         = "null";
  $rdf_image_data["flash_mode"]             = "null";

  */

  return $image_data;
}

function convert_rdf_lens_serial_number($database, $user_id, $serial_number) {
  $lens = pg_fetch_row(pg_query($database, "
     select identifier from lens
      where serial_number=$serial_number
        and users=$user_id"));
  if ($lens)
    return $lens[0];

  return "null";
}

?>
