<?php

//  Copyright (C) 2002-2006 Balint Kis (balint@k-i-s.net)
//  Copyright (C) 2005-2010 Solomon Peachy (pizza@shaftnet.org)

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

include_once "stemming.php";

$search_interface = array();

function emit_date_field_combo($item_name, $value) {
  global $strings;
  $val = "";

  $val .= "<select name=\"$item_name\">\n";
  $val .= emit_option('photo.date_of_exposure', $strings['generic_exposed'], $value == 'photo.date_of_exposire');
  $val .= emit_option('photo.date_added', $strings['generic_imported'], $value == 'photo.date_added');
  $val .= "</select>\n";
  return $val;
}

function search_i18n_data() {
  global $strings;
  global $search_interface;

  $search_interface = array("text_general"  => array("idx" => 1, 'link' => "search.text.general.php",  "alt" => $strings['search_gen_text']),
			  "text_advanced" => array("idx" => 2, 'link' => "search.text.advanced.php", "alt" => $strings['search_adv_text']));

  $search_interface["folder"] = array("idx" => 3, 'link' => generate_link('search.folder', ''), 'alt' => $strings['search_folders']);
  $search_interface["users"] = array("idx" => 4, 'link' => 'search.users.php', 'alt' => $strings['search_users']);
  $search_interface["tags"] = array("idx" => 5, 'link' => generate_link('tag', ''), 'alt' => $strings['search_tags']);
}

$generate_i18n_data[] = "search_i18n_data";

function build_itemized_search_string($search_string, $identifier) {
  $itemized_search_string = "";

  /* translate tabs and new lines into spaces */
  $token = strtok($search_string," ");
  while ($token) {
    if (strncmp($token, "AND", 3) == 0) {
      $itemized_search_string = $itemized_search_string . "AND ";
    } elseif (strncmp($token, "NOT", 3) == 0) {
      $itemized_search_string = $itemized_search_string . "NOT ";
    } elseif (strncmp($token, "OR", 2) == 0) {
      $itemized_search_string = $itemized_search_string . "OR ";
    } else {
      $itemized_search_string = $itemized_search_string . emit_a("search.text.general.php?$identifier=$token", "$token ");
    }
    $token = strtok("+ \n\t");
  }

  return $itemized_search_string;
}

function validate_search_string($search_string) {
  $search_string  = str_replace("\\", "", $search_string);
  $search_string  = str_replace("\"", "&quot;", $search_string);
  $search_string  = str_replace("\'", "&quot;", $search_string);
  $search_string  = str_replace("*",  "", $search_string);
  $search_string  = str_replace("?",  "", $search_string);
  return $search_string;
}

function prepare_keyword($keyword, $enable_stemming) {
  global $forbidden_keywords;
  
  $keyword = pg_escape_string($keyword);

  if (is_numeric($keyword))
    return " = '".$keyword."'";

  /* check for photo identifiers */
  $dash_position = strrpos($keyword, "-");
  if (is_integer($dash_position)) {
    $identifier = substr($keyword, 0, $dash_position);
    $identifier = ltrim($identifier, "0");
    if (is_numeric($identifier))
      return " = '".$identifier."'";
  }

  /* check if we have to search for exact maches */
  /* quotation marks must be at the beginning */
  /* and the end of the keyword, elsewhere ignored */
  $quot_1 = substr($keyword, 0, 6);
  $quot_2 = substr($keyword, strlen($keyword)-6, 6);
  if (($quot_1 == "&quot;") &&
      ($quot_2 == "&quot;")) {
    $keyword = str_replace("&quot;", "", $keyword);
    $keyword = str_replace(";", " ", $keyword);
    $keyword = str_replace("#@#", " ", $keyword);

    return " ILIKE '%$keyword%'";
  }

  /* ignore all signle quotation marks */
  $keyword = str_replace("&quot;", "", $keyword);

  /* prune forbidden keywords */
  foreach($forbidden_keywords as $forbidden_keyword) {
    if (!strcasecmp($forbidden_keyword, $keyword)) {
      return;
    }
  }

  /* stem keyword */
  if ($enable_stemming == 't') {
    $stemming_engine = new stemming();
    return $stemming_engine->stemstr($keyword);
  }

  return " ILIKE '%".$keyword."%'";
}

function extract_keywords($search_string, $enable_stemming) {
  /* detect quotation marks and replace spaces with the #@# marker so */
  /* that strings between quotation marks are treated as one keyword */
  /* when tokenizing the search string for operators */

  $inside_quotation = FALSE;
  $working_copy = $search_string;
  $search_string = "";
  while ($working_copy) {
    if ($inside_quotation) {
      $pos = strpos($working_copy, "&quot;");
      if ($pos === FALSE) {
        $search_string .= $working_copy;
        break;
      } else {
        $search_string .= str_replace(" ", "#@#", substr($working_copy, 0, $pos+5));
        $working_copy = substr($working_copy, $pos+5);
        $inside_quotation = FALSE;
      }
    } else {
      $pos = strpos($working_copy, "&quot;");
      if ($pos === FALSE) {
        $search_string .= $working_copy;
        break;
      } else {
        $search_string .= substr($working_copy, 0, $pos+5);
        $working_copy = substr($working_copy, $pos+5);
        $inside_quotation = TRUE;
      }
    }
  }

  /* tokenize the search string based on operators and spaces */
  $keyword = array();

  $token = strtok($search_string, "+ \n\t");
  while ($token !== FALSE) {
    switch(strtolower($token)) {
    case 'and':
      $token = strtok("+ \n\t");
      $token = prepare_keyword($token, $enable_stemming);
      if ($token)
	$keyword[] = array("operator"=>"AND", "keyword"=>$token);
      break;
    case 'not':
      $token = strtok("+ \n\t");
      $token = prepare_keyword($token, $enable_stemming);
      if ($token)
	$keyword[] = array("operator"=>"NOT", "keyword"=>$token);
      break;
    case 'or':
      $token = strtok("+ \n\t");
      $token = prepare_keyword($token, $enable_stemming);
      if ($token)
	$keyword[] = array("operator"=>"OR", "keyword"=>$token);
      break;
    default:
      $token = prepare_keyword($token, $enable_stemming);
      if ($token)
	$keyword[] = array("operator"=>"AND", "keyword"=>$token);
    }
    $token = strtok("+ \n\t");
  }
  if (!$keyword)
    return false;

  return $keyword;
}

function build_sql_search_string_keywords($keywords) {
  $string_a = " photo.identifier in (select k.photo from photo_keywords k where k.keyword";
    
  $sql_search_string = "";

  if (!$keywords)
    return $sql_search_string;

  foreach($keywords as $keyword) {
    switch (strtolower($keyword['operator'])) {
    case 'and':
      if ($sql_search_string == "")
        $sql_search_string = sprintf("%s %s)", $string_a, $keyword['keyword']);
      else
        $sql_search_string = sprintf("%s and %s %s)", $sql_search_string, $string_a, $keyword['keyword']);
      break;
    case 'not':
      if ($sql_search_string == "")
        $sql_search_string = sprintf("%s NOT %s)", $string_a, $keyword['keyword']);
      else
        $sql_search_string = sprintf("%s and NOT %s %s)", $sql_search_string, $string_a, $keyword['keyword']);
      break;
    case 'or':
      if ($sql_search_string == "")
        $sql_search_string = sprintf("%s %s)", $string_a, $keyword['keyword']);
      else
        $sql_search_string = sprintf("%s or %s %s)", $sql_search_string, $string_a, $keyword['keyword']);
      break;
    default:
      if ($sql_search_string == "")
        $sql_search_string = sprintf("%s %s)", $string_a, $keyword['keyword']);
      else
        $sql_search_string = sprintf("%s and %s %s)", $sql_search_string, $string_a, $keyword['keyword']);
    }
  }

  if ($sql_search_string != "")
    $sql_search_string = sprintf("(%s)", $sql_search_string);

  //  print "<pre>$sql_search_string</pre>";

  return "$sql_search_string";
}

function build_sql_search_string($keywords, $sql_field_name) {
  $sql_search_string = "";

  if (!$keywords)
    return $sql_search_string;

  foreach($keywords as $keyword) {
    switch(strtolower($keyword['operator'])) {
    case 'and':
      if ($sql_search_string == "")
        $sql_search_string = sprintf("%s %s", $sql_field_name, $keyword['keyword']);
      else
        $sql_search_string = sprintf("%s and %s %s", $sql_search_string, $sql_field_name, $keyword['keyword']);
      break;
    case 'not':
      if ($sql_search_string == "")
        $sql_search_string = sprintf("%s NOT %s", $sql_field_name, $keyword['keyword']);
      else
        $sql_search_string = sprintf("%s and NOT %s %s", $sql_search_string, $sql_field_name, $keyword['keyword']);
      break;
    case 'or':
      if ($sql_search_string == "")
        $sql_search_string = sprintf("%s %s", $sql_field_name, $keyword['keyword']);
      else
        $sql_search_string = sprintf("%s or %s %s", $sql_search_string, $sql_field_name, $keyword['keyword']);
      break;
    default:
      if ($sql_search_string == "")
        $sql_search_string = sprintf("%s %s", $sql_field_name, $keyword['keyword']);
      else
        $sql_search_string = sprintf("%s and %s %s", $sql_search_string, $sql_field_name, $keyword['keyword']);
    }
  }

  if ($sql_search_string != "")
    $sql_search_string = sprintf("(%s)", $sql_search_string);

  return $sql_search_string;
}

function build_folder_search_string($search_string, $identifier) {
  $itemized_search_string = "";

  /* translate tabs and new lines into spaces */
  $token = strtok($search_string," ");
  while ($token) {
    if (strncmp($token, "AND", 3) == 0) {
      $itemized_search_string = $itemized_search_string . "AND ";
    } elseif (strncmp($token, "NOT", 3) == 0) {
      $itemized_search_string = $itemized_search_string . "NOT ";
    } elseif (strncmp($token, "OR", 2) == 0) {
      $itemized_search_string = $itemized_search_string . "OR ";
    } else {
      $itemized_search_string = $itemized_search_string . emit_a(generate_link('search.folder', $token), "$token ");
    }
    $token = strtok("+ \n\t");
  }

  return $itemized_search_string;
}

?>
